@@ -38,14 +38,26 @@ tharea::tharea()
   this->place = TT_2DOBJ_PLACE_DEFAULT_BOTTOM;
   this->first_line = NULL;
   this->last_line = NULL;
+  this->m_outline_line = NULL;
 }
 
 
 tharea::~tharea()
 {
+  if (this->m_outline_line != NULL)
+    delete this->m_outline_line;
 }
 
 
+void tharea::start_insert() {
+  if (this->type == TT_AREA_TYPE_U) {
+    if (this->m_subtype_str == NULL)
+      ththrow(("missing subtype specification for area of user defined type"))
+  }
+}
+
+
+
 int tharea::get_class_id() 
 {
   return TT_AREA_CMD;
@@ -90,7 +102,7 @@ thcmd_option_desc tharea::get_cmd_option_desc(char * opts)
 
 void tharea::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long indataline)
 {
-  
+  char * type, * subtype;  
   if (cod.id == 1)
     cod.id = TT_AREA_TYPE;
     
@@ -103,9 +115,10 @@ void tharea::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
       break;
   
     case TT_AREA_TYPE:
-      this->type = thmatch_token(*args, thtt_area_types);
-      if (this->type == TT_AREA_TYPE_UNKNOWN)
-        ththrow(("unknown area type -- %s", *args))
+      th2dsplitTT(*args, &type, &subtype);
+      this->parse_type(type);
+      if (strlen(subtype) > 0)
+        this->parse_subtype(subtype);
       break;
     
     default:
@@ -115,6 +128,29 @@ void tharea::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
 }
 
 
+void tharea::parse_type(char * tstr)
+{
+  this->type = thmatch_token(tstr, thtt_area_types);
+  if (this->type == TT_AREA_TYPE_UNKNOWN)
+    ththrow(("unknown area type -- %s", tstr))
+  if (this->type == TT_AREA_TYPE_DIMENSIONS)
+    ththrow(("area dimensions is not supported as ordinary type"))
+}
+
+
+
+void tharea::parse_subtype(char * ststr)
+{
+  if (this->type == TT_AREA_TYPE_UNKNOWN)
+    ththrow(("area type must be specified before subtype"))
+  if (this->type == TT_AREA_TYPE_U) {
+    this->parse_u_subtype(ststr);
+    return;
+  } else
+    ththrow(("invalid type - subtype combination"))
+}
+
+
 void tharea::self_delete()
 {
   delete this;
@@ -175,6 +211,10 @@ bool tharea::export_mp(class thexpmapmpxs * out)
     tharea_type_export_mp(TT_AREA_TYPE_ICE, SYMA_ICE)
     tharea_type_export_mp(TT_AREA_TYPE_CLAY, SYMA_CLAY)
     tharea_type_export_mp(TT_AREA_TYPE_PEBBLES, SYMA_PEBBLES)
+    tharea_type_export_mp(TT_AREA_TYPE_BEDROCK, SYMA_BEDROCK)
+    tharea_type_export_mp(TT_AREA_TYPE_FLOWSTONE, SYMA_FLOWSTONE)
+    tharea_type_export_mp(TT_AREA_TYPE_MOONMILK, SYMA_MOONMILK)
+    tharea_type_export_mp(TT_AREA_TYPE_U, SYMA_U)
   }
   
   if (this->context >= 0) 
@@ -187,9 +227,10 @@ bool tharea::export_mp(class thexpmapmpxs * out)
   if (out->file == NULL)
     return(true);
     
+  th2ddataobject::export_mp(out);
   thdb_revision_set_type::iterator ri = 
       this->db->revision_set.find(threvision(this->id, 0));
-  fprintf(out->file,"current_src := \"%s [%d]\";\n", ri->srcf.name, ri->srcf.line);  
+  fprintf(out->file,"current_src := \"%s [%ld]\";\n", ri->srcf.name, ri->srcf.line);  
   fprintf(out->file,"string area_border[];\n");
   thdb2dab * bl = this->first_line;
   int blnum = 1;
@@ -208,7 +249,10 @@ bool tharea::export_mp(class thexpmapmpxs * out)
     bl = bl->next_line;
   }
   
-  fprintf(out->file,"));\n");
+  if (this->type == TT_AREA_TYPE_U)
+    fprintf(out->file,"),\"%s\");\n", (this->m_subtype_str == NULL) ? "" : this->m_subtype_str);
+  else
+    fprintf(out->file,"));\n");
   
   return(false);  
   
