@@ -58,6 +58,7 @@ static const thstok thtt_area_opt[] = {
  
 enum {
   TT_AREA_TYPE_UNKNOWN,
+  TT_AREA_TYPE_U,
   TT_AREA_TYPE_WATER,
   TT_AREA_TYPE_SUMP,
   TT_AREA_TYPE_SAND,
@@ -67,6 +68,10 @@ enum {
   TT_AREA_TYPE_ICE,
   TT_AREA_TYPE_CLAY,
   TT_AREA_TYPE_PEBBLES,
+  TT_AREA_TYPE_BEDROCK,
+  TT_AREA_TYPE_FLOWSTONE,
+  TT_AREA_TYPE_MOONMILK,
+  TT_AREA_TYPE_DIMENSIONS,
 };
 
 
@@ -75,14 +80,19 @@ enum {
  */
  
 static const thstok thtt_area_types[] = {
+  {"bedrock", TT_AREA_TYPE_BEDROCK},
   {"blocks", TT_AREA_TYPE_BLOCKS},
   {"clay", TT_AREA_TYPE_CLAY},
   {"debris", TT_AREA_TYPE_DEBRIS},
+  {"dimensions", TT_AREA_TYPE_DIMENSIONS},
+  {"flowstone", TT_AREA_TYPE_FLOWSTONE},
   {"ice", TT_AREA_TYPE_ICE},
+  {"moonmilk", TT_AREA_TYPE_MOONMILK},
   {"pebbles", TT_AREA_TYPE_PEBBLES},
   {"sand", TT_AREA_TYPE_SAND},
   {"snow", TT_AREA_TYPE_SNOW},
   {"sump", TT_AREA_TYPE_SUMP},
+  {"u",TT_AREA_TYPE_U},
   {"water", TT_AREA_TYPE_WATER},
   {NULL, TT_AREA_TYPE_UNKNOWN},
 };
@@ -102,6 +112,8 @@ class tharea : public th2ddataobject {
   
   thdb2dab * first_line,  ///< First border line.
     * last_line;  ///< Last border line.
+
+  class thline * m_outline_line;
     
   void insert_border_line(int npars, char ** pars);  ///< Insert border line.
 
@@ -202,7 +214,14 @@ class tharea : public th2ddataobject {
    */
    
   virtual bool export_mp(class thexpmapmpxs * out);
+
+
+  void parse_type(char * tstr);  ///< Parse area type.
   
+  void parse_subtype(char * ststr);  ///< Parse area subtype.
+
+  virtual void start_insert();
+
 
 };
 
