@@ -40,6 +40,7 @@
 #include "thdb2d.h"
 #include "thobjectname.h"
 #include "thobjectsrc.h"
+#include "thattr.h"
 #include <stdio.h>
 
 /**
@@ -164,6 +165,7 @@ class thdatabase {
   int ccontext;  ///< Current context.
   class thsurvey * csurveyptr,  ///< Pointer to the current survey.
     * fsurveyptr;  ///< Pointer to the first survey.
+  unsigned csurveylevel; ///< Current survey level.
   class thscrap * cscrapptr; ///< Current scrap.
   class th2ddataobject * lcscrapoptr; ///< Last object in given current scrap.
   class thdataobject * lcsobjectptr;  ///< Last object in given current survey.
@@ -184,6 +186,7 @@ class thdatabase {
   thdb_revision_set_type revision_set;  ///< Object revisions.
   thdb1d db1d;  ///< Survey data part of database.
   thdb2d db2d;  ///< 2D data part of database.
+  thattr attr;  ///< Object attributes.
   thobjectsrc csrc;  ///< Current source position.
   thdb_object_map_type object_map;  ///< Object search hash
   thdb_survey_map_type survey_map;   ///< Survey search hash
@@ -288,6 +291,7 @@ class thdatabase {
    */
    
   char * strstore(char * src, bool use_dic = false);
+  char * strstore(const char * src, bool use_dic = false);
   
   
   /**
@@ -324,6 +328,7 @@ class thdatabase {
    
   thsurvey * get_survey(char * sn, thsurvey * ps);
   thsurvey * get_survey_noexc(char * sn, thsurvey * ps);
+  thsurvey * get_exact_survey_noexc(char * sn, thsurvey * ps);
 
 
   /**
