@@ -73,6 +73,8 @@ class thdb2dprjpr {
 class thdb2d {
 
   public:
+
+  bool processed_area_outlines;
   
   class thdatabase * db;  ///< Our database.
   thdb2dprj_list prj_list;  ///< Set of projections.
@@ -105,15 +107,17 @@ class thdb2d {
   void pp_calc_limits(thdb2dprj * prj);  ///< ???
   void pp_calc_stations(thdb2dprj * prj);  ///< ???
   void pp_adjust_points(thdb2dprj * prj);  ///< ???
-  void pp_shift_points(thdb2dprj * prj, bool calc_az = false);  ///< ???
+  void pp_shift_points(thdb2dprj * prj);  ///< ???
+  void pp_morph_points(thdb2dprj * prj);
+  void pp_calc_points_z(thdb2dprj * prj);
   void pp_process_joins(thdb2dprj * prj); ///< ???
   bool pp_process_adjustments(thdb2dprj * prj);
   void pp_smooth_lines(thdb2dprj * prj);  ///< ???
   void pp_smooth_joins(thdb2dprj * prj);  ///< ???
   void pp_calc_distortion(thdb2dprj * prj);  ///< ???
 
-  void insert_basic_maps(thdb2dxm * fmap, thmap * map, int mode, int level); ///< ???
-  thdb2dxm * insert_maps(thdb2dxm * selection,thdb2dxm * insert_after,thmap * map, 
+  void insert_basic_maps(thdb2dxm * fmap, thmap * map, int mode, int level, thdb2dmi_shift shift = thdb2dmi_shift()); ///< ???
+  thdb2dxm * insert_maps(thdb2dxm * selection,thdb2dxm * insert_after,thmap * map,
     unsigned long selection_level, int level, int title_level, int map_level); ///< ???
   void reset_selection();
     
@@ -122,6 +126,8 @@ class thdb2d {
   thmbuffer mbf,  ///< Multi buffer.
     mbf2; ///< Second buffer.
   thbuffer bf; /// Buffer.
+
+  thdb2dxs_target_map m_target_map;
     
   /**
    * Standard constructor.
@@ -266,6 +272,13 @@ class thdb2d {
   void process_projection(thdb2dprj * prj);
   
   
+  /**
+   * Process projection.
+   */
+   
+  void process_areas_in_projection(thdb2dprj * prj);
+  
+  
   /**
    * Make map selection.
    */
