@@ -33,6 +33,7 @@
 #include "thexception.h"
 #include "thtexfonts.h"
 #include "thlang.h"
+#include "thlocale.h"
 
 #ifdef THWIN32
 #include <windows.h>
@@ -41,11 +42,17 @@
 const char * THCCC_INIT_FILE = "### Output character encodings ###\n"
 "# encoding-default  ASCII\n"
 "# encoding-sql  ASCII\n\n"
-"### Default output language ###\n"
-"# language  en_UK\n\n"
+"### Default regional settings ###\n"
+"# language  en_UK\n"
+"# units metric\n\n"
+"### Prefered loop closure method\n"
+"# loop-closure survex\n\n"
 "### Paths to called executable files ###\n"
 "# mpost-path  \"mpost\"\n"
-"# pdftex-path  \"pdfetex\"\n\n"
+"# pdftex-path  \"pdfetex\"\n"
+"# cavern-path  \"cavern\"\n"
+"# convert-path  \"convert\"\n"
+"# identify-path  \"identify\"\n\n"
 "### Search paths for source and configuration files ###\n"
 "# source-path  \"\"\n\n"
 "### Tex initialization ###\n"
@@ -61,6 +68,7 @@ const char * THCCC_INIT_FILE = "### Output character encodings ###\n"
 thinit::thinit()
 {
   this->tex_env = false;
+	this->loopc = THINIT_LOOPC_UNKNOWN;
 }
 
 
@@ -68,11 +76,14 @@ thinit::~thinit()
 {
 }
 
+
 enum {
   TTIC_ENCODING_DEFAULT,
   TTIC_ENCODING_SQL,
   TTIC_PATH_CAVERN,
 //  TTIC_PATH_3DTOPOS,
+  TTIC_PATH_CONVERT,
+  TTIC_PATH_IDENTIFY,
   TTIC_PATH_MPOST,
   TTIC_PATH_PDFTEX,
   TTIC_PATH_SOURCE,
@@ -81,6 +92,8 @@ enum {
   TTIC_LANG,
   TTIC_TEX_FONTS,
   TTIC_TEX_ENV,
+	TTIC_UNITS,
+	TTIC_LOOPC,
   TTIC_UNKNOWN,
 };
 
@@ -90,12 +103,15 @@ enum {
  */
  
 static const thstok thtt_initcmd[] = {
-//  {"cavern-path", TTIC_PATH_CAVERN},
+  {"cavern-path", TTIC_PATH_CAVERN},
+  {"convert-path", TTIC_PATH_CONVERT},
   {"encoding-default", TTIC_ENCODING_DEFAULT},
   {"encoding-sql", TTIC_ENCODING_SQL},
 //  {"encoding_default", TTIC_ENCODING_DEFAULT},
 //  {"path_3dtopos", TTIC_PATH_3DTOPOS},
+  {"identify-path", TTIC_PATH_IDENTIFY},
   {"language", TTIC_LANG},
+  {"loop-closure", TTIC_LOOPC},
   {"mpost-path", TTIC_PATH_MPOST},
   {"pdftex-path", TTIC_PATH_PDFTEX},
   {"source-path", TTIC_PATH_SOURCE},
@@ -103,6 +119,7 @@ static const thstok thtt_initcmd[] = {
   {"tex-fonts",TTIC_TEX_FONTS},
   {"tmp-path",TTIC_TMP_PATH},
   {"tmp-remove",TTIC_TMP_REMOVE_SCRIPT},
+  {"units",TTIC_UNITS},
   {NULL, TTIC_UNKNOWN},
 };
 
@@ -116,6 +133,32 @@ void thinit__print_open(char * s) {
 #endif 
 }
 
+
+static const thstok thtt_loopc[] = {
+  {"survex", THINIT_LOOPC_SURVEX},
+  {"therion", THINIT_LOOPC_THERION},
+  {NULL, THINIT_LOOPC_UNKNOWN},
+};
+
+
+#ifdef THWIN32
+thbuffer short_path_buffer;
+void thinit__make_short_path(thbuffer * bf) {
+  DWORD rv;
+  size_t sl;
+  sl = strlen(bf->get_buffer());
+  if (sl == 0)
+    return;
+  short_path_buffer.guarantee(2 * sl);
+  rv = GetShortPathName(bf->get_buffer(), short_path_buffer.get_buffer(), (DWORD) short_path_buffer.size);
+  if ((rv > 0) && (rv < (DWORD) short_path_buffer.size)) {
+    bf->strcpy(short_path_buffer.get_buffer());
+  }
+}
+#endif
+
+
+
 void thinit::load()
 {
 
@@ -130,6 +173,7 @@ void thinit::load()
   // set cavern path according to Windows registers
   this->path_cavern.guarantee(1024);
   thmbuffer mbf;
+  thbuffer bf;
   DWORD type, length = 1024;
   HKEY key;
   bool loaded_ok = true;
@@ -145,14 +189,28 @@ void thinit::load()
     loaded_ok = false;
   if (loaded_ok) {
     thsplit_args(&mbf,this->path_cavern.get_buffer());
-    this->path_cavern = "\"";
-    this->path_cavern += *(mbf.get_buffer());
-    this->path_cavern += "\"";
+    this->path_cavern = *(mbf.get_buffer());
   } else {
     this->path_cavern = "cavern";
   }
 #endif  
 
+	// try running survex
+	if (this->loopc == THINIT_LOOPC_UNKNOWN) {
+	  thbuffer svxcom;
+	  svxcom = "\"";
+	  svxcom += thini.get_path_cavern();
+	  svxcom += "\" --version";  
+#ifdef THDEBUG
+	  thprintf("testing cavern\n");
+#endif
+	  if (system(svxcom.get_buffer()) == EXIT_SUCCESS) {
+			this->loopc = THINIT_LOOPC_SURVEX;
+		} else {
+			this->loopc = THINIT_LOOPC_THERION;
+		}
+	}
+
 //  this->path_3dtopos = "3dtopos";
 #ifdef THWIN32
   if (thcfg.install_tex) {
@@ -167,6 +225,21 @@ void thinit::load()
 #ifdef THWIN32
   }
 #endif  
+
+#ifdef THWIN32
+  if (thcfg.install_im) {
+    this->path_convert = thcfg.install_path.get_buffer();
+    this->path_convert += "\\bin\\convert.exe";
+    this->path_identify = thcfg.install_path.get_buffer();
+    this->path_identify += "\\bin\\identify.exe";
+  } else {
+#endif  
+    this->path_convert = "convert";
+    this->path_identify = "identify";
+#ifdef THWIN32
+  }
+#endif  
+
   this->tmp_path = "";
   this->tmp_remove_script = "";
   this->lang = THLANG_UNKNOWN;
@@ -193,8 +266,12 @@ void thinit::load()
       switch (argid) {      
         case TTIC_ENCODING_DEFAULT:
         case TTIC_PATH_CAVERN:
+        case TTIC_PATH_CONVERT:
+        case TTIC_PATH_IDENTIFY:
+        case TTIC_LOOPC:
         case TTIC_TMP_PATH:
         case TTIC_LANG:
+        case TTIC_UNITS:
         case TTIC_TMP_REMOVE_SCRIPT:
         case TTIC_PATH_MPOST:
         case TTIC_PATH_PDFTEX:
@@ -226,6 +303,18 @@ void thinit::load()
             ththrow(("invalid path"))
           this->path_cavern.strcpy(args[1]);
           break;
+          
+        case TTIC_PATH_CONVERT:
+          if (strlen(args[1]) < 1)
+            ththrow(("invalid path"))
+          this->path_convert.strcpy(args[1]);
+          break;
+          
+        case TTIC_PATH_IDENTIFY:
+          if (strlen(args[1]) < 1)
+            ththrow(("invalid path"))
+          this->path_identify.strcpy(args[1]);
+          break;
 
         case TTIC_TMP_PATH:
           this->tmp_path.strcpy(args[1]);
@@ -237,6 +326,10 @@ void thinit::load()
             ththrow(("language not supported -- %s",args[1]))
           break;
 
+        case TTIC_UNITS:
+					thdeflocale.parse_units(args[1]);
+          break;
+
         case TTIC_TMP_REMOVE_SCRIPT:
           this->tmp_remove_script.strcpy(args[1]);
           break;
@@ -277,6 +370,13 @@ void thinit::load()
           this->tex_env = (sv == TT_TRUE);
           break;
           
+        case TTIC_LOOPC:
+          sv = thmatch_token(args[1], thtt_loopc);
+          if (sv == THINIT_LOOPC_UNKNOWN)
+            ththrow(("invalid loop-closure switch -- %s", args[1]))
+          this->loopc = sv;
+          break;
+          
         default:
           ththrow(("invalid initialization command -- %s", args[0]))
       }
@@ -294,6 +394,15 @@ void thinit::load()
   }
   if (!some_tex_fonts)
      init_encodings();
+
+#ifdef THWIN32
+  thinit__make_short_path(&this->path_cavern);
+  thinit__make_short_path(&this->path_convert);
+  thinit__make_short_path(&this->path_identify);
+  thinit__make_short_path(&this->path_mpost);
+  thinit__make_short_path(&this->path_pdftex);
+#endif
+
 }
 
 
@@ -302,6 +411,18 @@ char * thinit::get_path_cavern()
   return this->path_cavern.get_buffer();
 }
 
+
+char * thinit::get_path_convert()
+{
+  return this->path_convert.get_buffer();
+}
+
+
+char * thinit::get_path_identify()
+{
+  return this->path_identify.get_buffer();
+}
+
 //char * thinit::get_path_3dtopos()
 //{
 //  return this->path_3dtopos.get_buffer();
