@@ -35,6 +35,7 @@
 #include "thtexfonts.h"
 #include "thdate.h"
 #include "thscrap.h"
+#include <string>
 
 
 thpoint::thpoint()
@@ -47,9 +48,7 @@ thpoint::thpoint()
   this->point->pscrap = thdb.get_current_scrap();
   
   this->station_name.clear();
-  this->extend_name.clear();
-  this->extend_point = NULL;
-  this->extend_opts = TT_POINT_EXTEND_NONE;
+  this->from_name.clear();
   
   this->orient = thnan;
   this->xsize = thnan;
@@ -57,6 +56,7 @@ thpoint::thpoint()
   this->align = TT_POINT_ALIGN_C;
   
   this->text = NULL;
+  this->code = NULL;
 }
 
 
@@ -116,11 +116,20 @@ thcmd_option_desc thpoint::get_cmd_option_desc(char * opts)
 }
 
 
+void thpoint::start_insert() {
+  if (this->type == TT_POINT_TYPE_U) {
+    if (this->m_subtype_str == NULL)
+      ththrow(("missing subtype specification for point of user defined type"))
+  }
+}
+
+
 void thpoint::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long indataline)
 {
 
   double dv;
   int sv;
+  char * type, * subtype;
   
   if (cod.id == 3)
     cod.id = TT_POINT_TYPE;
@@ -139,18 +148,30 @@ void thpoint::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
       break;
 
     case TT_POINT_TYPE:
-      this->parse_type(*args);
+      th2dsplitTT(*args, &type, &subtype);
+      this->parse_type(type);
+      if (strlen(subtype) > 0)
+        this->parse_subtype(subtype);
       break;
 
     case TT_POINT_VALUE:
       this->parse_value(*args);
       break;
 
+    case TT_POINT_DIST:
+      this->parse_value(*args);
+      break;
+
     case TT_POINT_TEXT:
       thencode(&(this->db->buff_enc), *args, argenc);
       this->parse_text(this->db->buff_enc.get_buffer());
       break;
 
+    case TT_POINT_CODE:
+      thencode(&(this->db->buff_enc), *args, argenc);
+      this->parse_code(this->db->buff_enc.get_buffer());
+      break;
+
     case TT_POINT_ALIGN:
       this->parse_align(*args);
       break;
@@ -206,13 +227,18 @@ void thpoint::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
       break;
       
     case TT_POINT_STATION:
-      if (this->type != TT_POINT_TYPE_STATION)
-        ththrow(("point not station -- -name"))
-      thparse_objectname(this->station_name, & this->db->buff_stations, *args, thdb.cscrapptr);
+      switch (this->type) {
+        case TT_POINT_TYPE_STATION:
+        case TT_POINT_TYPE_CONTINUATION:
+          thparse_objectname(this->station_name, & this->db->buff_stations, *args, thdb.cscrapptr);
+          break;
+        default:
+          ththrow(("station reference not allowed with this point type"))
+      }
       break;
       
-    case TT_POINT_EXTEND:
-      this->parse_extend(*args);
+    case TT_POINT_FROM:
+      this->parse_from(*args);
       break;
     
     // if not found, try to set fathers properties  
@@ -226,6 +252,7 @@ void thpoint::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
           case TT_POINT_TYPE_ALTITUDE:
           case TT_POINT_TYPE_HEIGHT:
           case TT_POINT_TYPE_DIMENSIONS:
+          case TT_POINT_TYPE_MAP_CONNECTION:
           case TT_POINT_TYPE_PASSAGE_HEIGHT:
             ththrow(("-clip not valid with type %s", thmatch_string(this->type,thtt_point_types)))
             break;
@@ -256,9 +283,8 @@ void thpoint::self_print_properties(FILE * outf)
   if (this->type == TT_POINT_TYPE_STATION) {
     fprintf(outf,"\tstation: ");
     fprintf(outf,this->station_name);
-    fprintf(outf,"\n\textend station: ");
-    fprintf(outf,this->extend_name);
-    fprintf(outf,"\n\textend opts: %d\n",this->extend_opts);
+    fprintf(outf,"\n\tfrom station: ");
+    fprintf(outf,this->from_name);
   }
   // insert intended print of object properties here
 }
@@ -288,6 +314,10 @@ void thpoint::parse_subtype(char * ststr)
 {
   if (this->type == TT_POINT_TYPE_UNKNOWN)
     ththrow(("point type must be specified before subtype"))
+  if (this->type == TT_POINT_TYPE_U) {
+    this->parse_u_subtype(ststr);
+    return;
+  }
   this->subtype = thmatch_token(ststr, thtt_point_subtypes);
   if (this->subtype == TT_POINT_SUBTYPE_UNKNOWN)
     ththrow(("unknown point subtype -- %s", ststr))
@@ -304,6 +334,15 @@ void thpoint::parse_subtype(char * ststr)
           break;
       }
       break;
+    case TT_POINT_TYPE_AIR_DRAUGHT:
+      switch (this->subtype) {
+        case TT_POINT_SUBTYPE_UNDEF:
+        case TT_POINT_SUBTYPE_WINTER:
+        case TT_POINT_SUBTYPE_SUMMER:
+          combok = true;
+          break;
+      }
+      break;
     case TT_POINT_TYPE_WATER_FLOW:
       switch (this->subtype) {
         case TT_POINT_SUBTYPE_PERMANENT:
@@ -318,55 +357,14 @@ void thpoint::parse_subtype(char * ststr)
     ththrow(("invalid point type - subtype combination"))
 }
 
-void thpoint::parse_extend(char * estr)
+void thpoint::parse_from(char * estr)
 {
   thsplit_words(& this->db->db2d.mbf, estr);
-  int npar = this->db->db2d.mbf.get_size(),cpar,stic;
+  int npar = this->db->db2d.mbf.get_size();
   char ** pars = this->db->db2d.mbf.get_buffer();
-  if (npar < 1)
-    ththrow(("no -extend arguments"))
-  cpar = 0;
-  while (cpar < npar) {
-    switch (thmatch_token(pars[cpar],thtt_point_extopt)) {
-      case TT_POINT_EXTEND_LEFT:
-        this->extend_opts = this->extend_opts & 
-          (~(TT_POINT_EXTEND_LEFT | TT_POINT_EXTEND_RIGHT));
-        this->extend_opts = this->extend_opts | TT_POINT_EXTEND_LEFT;
-        break;  
-      case TT_POINT_EXTEND_RIGHT:
-        this->extend_opts = this->extend_opts & 
-          (~(TT_POINT_EXTEND_LEFT | TT_POINT_EXTEND_RIGHT));
-        this->extend_opts = this->extend_opts | TT_POINT_EXTEND_RIGHT;
-        break;
-      case TT_POINT_EXTEND_ROOT:
-        this->extend_opts = this->extend_opts | TT_POINT_EXTEND_ROOT;
-        break;
-      case TT_POINT_EXTEND_STICKY:
-        cpar++;
-        if (cpar == npar)
-          ththrow(("logical value expected"))
-        stic = thmatch_token(pars[cpar],thtt_bool);
-        if (stic == TT_UNKNOWN_BOOL)
-          ththrow(("not a logical value -- %s", pars[cpar]))
-        this->extend_opts = this->extend_opts & 
-          (~(TT_POINT_EXTEND_STICKYON | TT_POINT_EXTEND_STICKYOFF));
-        if (stic == TT_TRUE)
-          this->extend_opts = this->extend_opts | TT_POINT_EXTEND_STICKYON;
-        else
-          this->extend_opts = this->extend_opts | TT_POINT_EXTEND_STICKYOFF;
-        break;
-      case TT_POINT_EXTEND_PREV:
-        cpar++;
-        if (cpar == npar)
-          ththrow(("station name expected"))
-//          ththrow(("point or station name expected"))
-        thparse_objectname(this->extend_name,& this->db->buff_stations,pars[cpar],this->fscrapptr);
-        break;
-      default:
-        ththrow(("invalid keyword -- %s",pars[cpar]))
-    }
-    cpar++;
-  }
+  if (npar != 1)
+    ththrow(("invalid from station reference -- %s", estr))
+  thparse_objectname(this->from_name,& this->db->buff_stations,pars[0],this->fscrapptr);
 }
 
 
@@ -396,7 +394,10 @@ char * thpoint_export_mp_align2mp(int a) {
 
 bool thpoint::export_mp(class thexpmapmpxs * out)
 {
+  th2ddataobject::export_mp(out);
+
   bool postprocess = true;
+  std::string attr_text;
   int macroid = -1;
   int postprocess_label = -1;
   this->db->buff_enc.guarantee(8128);
@@ -406,6 +407,8 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
   switch(this->type) {
 
     case TT_POINT_TYPE_DIMENSIONS:
+    case TT_POINT_TYPE_EXTRA:
+    case TT_POINT_TYPE_MAP_CONNECTION:
       postprocess = false;
       break;
     case TT_POINT_TYPE_LABEL:  
@@ -680,8 +683,36 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
       }
       break;
 
+    case TT_POINT_TYPE_AIR_DRAUGHT:
+      switch (this->subtype) {
+        thpoint_type_export_mp(TT_POINT_SUBTYPE_WINTER,SYMP_AIRDRAUGHT_WINTER);
+        thpoint_type_export_mp(TT_POINT_SUBTYPE_SUMMER,SYMP_AIRDRAUGHT_SUMMER);
+        default:
+          macroid = SYMP_AIRDRAUGHT;
+      }
+      break;
+
+    case TT_POINT_TYPE_CONTINUATION:
+      macroid = SYMP_CONTINUATION;
+      {
+        std::string tmp;
+        if ((this->code != NULL) && (strlen(this->code) > 0)) {
+          tmp += this->code;
+        }
+        if ((this->text != NULL) && (strlen(this->text) > 0)) {
+          if (tmp.length() > 0)
+            tmp += " -- ";
+          tmp += this->text;
+        }
+        if (tmp.length() > 0) {
+          attr_text = "btex \\thcontinuation ";
+          attr_text += utf2tex(tmp);
+          attr_text += "etex";
+        }
+      }
+      break;
+
 // specialne symboly
-    thpoint_type_export_mp(TT_POINT_TYPE_AIR_DRAUGHT,SYMP_AIRDRAUGHT)
     thpoint_type_export_mp(TT_POINT_TYPE_SPRING,SYMP_SPRING)
     thpoint_type_export_mp(TT_POINT_TYPE_SINK,SYMP_SINK)
     thpoint_type_export_mp(TT_POINT_TYPE_ENTRANCE,SYMP_ENTRANCE)
@@ -699,7 +730,6 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
     thpoint_type_export_mp(TT_POINT_TYPE_CAMP,SYMP_CAMP)
 
 // ukoncenia chodby
-    thpoint_type_export_mp(TT_POINT_TYPE_CONTINUATION,SYMP_CONTINUATION)
     thpoint_type_export_mp(TT_POINT_TYPE_NARROW_END,SYMP_NARROWEND)
     thpoint_type_export_mp(TT_POINT_TYPE_LOW_END,SYMP_LOWEND)
     thpoint_type_export_mp(TT_POINT_TYPE_FLOWSTONE_CHOKE,SYMP_FLOWSTONECHOKE)
@@ -748,6 +778,8 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
     thpoint_type_export_mp(TT_POINT_TYPE_PALEO_MATERIAL,SYMP_PALEOMATERIAL)
     thpoint_type_export_mp(TT_POINT_TYPE_VEGETABLE_DEBRIS,SYMP_VEGETABLEDEBRIS)
     thpoint_type_export_mp(TT_POINT_TYPE_ROOT,SYMP_ROOT)
+
+    thpoint_type_export_mp(TT_POINT_TYPE_U,SYMP_U)
       
     default:
       macroid = SYMP_UNDEFINED;
@@ -761,10 +793,14 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
   
   if (this->context >= 0) 
     macroid = this->context;
+
   if ((macroid > 0) && postprocess) {
     if (out->symset->assigned[macroid]) {
       if (out->file == NULL)
         return(true);
+      if (attr_text.length() > 0) {
+        fprintf(out->file, "ATTR__text_x := true;\nATTR__text := %s;\n", attr_text.c_str());
+      }
       fprintf(out->file,"%s(",out->symset->get_mp_macro(macroid));
     }
     else
@@ -815,8 +851,14 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
         al = "(1,1)";
         break;
     }
-    fprintf(out->file,",%.1f,%.2f,%s);\n",
+    fprintf(out->file,",%.1f,%.2f,%s",
         (thisnan(this->orient) ? 0 : 360 - this->orient - out->rr),scl,al);
+    if (this->type == TT_POINT_TYPE_U) {
+      fprintf(out->file, ",\"%s\");\n", (this->m_subtype_str == NULL) ? "" : this->m_subtype_str);
+    } else {
+      fprintf(out->file, ");\n");
+    }
+
     if (out->layout->is_debug_stations() || out->layout->is_debug_joins()) {
       fprintf(out->file,"p_debug(-1,0,");
       this->point->export_mp(out);
@@ -832,6 +874,9 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
       this->point->export_mp(out,0);
       fprintf(out->file,");\n");
     }
+    if (attr_text.length() > 0) {
+      fprintf(out->file, "ATTR__text_x := false;\n");
+    }
   }
   
   return(false);
@@ -854,6 +899,7 @@ void thpoint::parse_text(char * ss) {
     case TT_POINT_TYPE_LABEL:
     case TT_POINT_TYPE_REMARK:
     case TT_POINT_TYPE_STATION_NAME:
+    case TT_POINT_TYPE_CONTINUATION:
       break;
     default:
       ththrow(("-text not valid with type %s", thmatch_string(this->type,thtt_point_types)))
@@ -861,6 +907,21 @@ void thpoint::parse_text(char * ss) {
   }
   if (strlen(ss) > 0)
     this->text = this->db->strstore(ss);
+  else
+    this->text = NULL;
+}
+
+
+void thpoint::parse_code(char * ss) {
+  switch (this->type) {
+    case TT_POINT_TYPE_CONTINUATION:
+      break;
+    default:
+      ththrow(("-code not valid with type %s", thmatch_string(this->type,thtt_point_types)))
+      break;
+  }
+  if (strlen(ss) > 0)
+    this->code = this->db->strstore(ss);
 }
 
 
@@ -921,6 +982,7 @@ void thpoint::parse_value(char * ss) {
     case TT_POINT_TYPE_PASSAGE_HEIGHT:
     case TT_POINT_TYPE_DIMENSIONS:
     case TT_POINT_TYPE_DATE:
+    case TT_POINT_TYPE_EXTRA:
       break;
     default:
       ththrow(("-value not valid with type %s", thmatch_string(this->type,thtt_point_types)))
@@ -938,6 +1000,29 @@ void thpoint::parse_value(char * ss) {
   thdate * dp;
   
   switch (this->type) {
+
+    case TT_POINT_TYPE_EXTRA:
+      ux = 0;
+      parsev = false;
+      switch (npar) {
+        case 1:
+          break;
+        case 2:
+          ux = 1;
+          break;
+        default:
+          ththrow(("invalid distance -- %s",ss))
+      }
+      this->xsize = thnan;
+      thparse_double(sv,dv,pars[0]);
+      if (sv != TT_SV_NUMBER)
+        ththrow(("not a number -- %s", pars[0]))
+      if (ux > 0) {
+        lentf.parse_units(pars[ux]);
+        dv = lentf.transform(dv);
+      }
+      this->xsize = dv;
+      break;
   
     // let's parse altitude
     case TT_POINT_TYPE_ALTITUDE:
@@ -1097,4 +1182,107 @@ void thpoint::parse_value(char * ss) {
 }
 
 
+void thpoint::check_extra()
+{
+  thdb2dcp * cp;
+  if (this->from_name.id == 0) {
+    double mind = -1.0, cd;
+    cp = this->fscrapptr->fcpp;
+    while (cp != NULL) {
+      if (cp->st != NULL) {
+        cd = thvec2(this->point->x - cp->pt->x, this->point->y - cp->pt->y).length();
+        if (mind < 0.0) {
+          this->from_name.id = cp->st->uid;
+          mind = cd;
+        } else if (cd < mind) {
+          this->from_name.id = cp->st->uid;
+          mind = cd;
+        }
+      }
+      cp = cp->nextcp;
+    }
+  }
+  if (thisnan(this->xsize) && (this->from_name.id > 0)) {
+    size_t myuid;
+    thdb2dcp * mycp;
+    myuid = thdb.db1d.station_vec[this->from_name.id - 1].uid;
+    cp = this->fscrapptr->fcpp;
+    mycp = NULL;
+    double cdist(0.0);
+    size_t ccount(0);
+    while (cp != NULL) {
+      if ((cp->st != NULL) && (cp->st->uid == myuid)) {
+        mycp = cp;
+        break;
+      }
+      cp = cp->nextcp;
+    }
+    if (mycp == NULL)
+      return;
+    thdb1d_tree_node * nodes = thdb.db1d.get_tree_nodes();
+    thdb1d_tree_arrow * carrow;
+    cp = this->fscrapptr->fcpp;
+    while (cp != NULL) {
+      if ((cp->st != NULL) && (cp->st->uid != myuid)) {
+        carrow = nodes[myuid - 1].first_arrow;
+        while (carrow != NULL) {
+          if (carrow->end_node->uid == cp->st->uid) {
+            bool reverse = false;
+            if (carrow->is_reversed) reverse = !reverse;
+            if (carrow->leg->reverse) reverse = !reverse;
+            thdataleg * cl = carrow->leg->leg;
+            thline2 xl;
+            if (reverse) {
+              xl = thline2(
+              thvec2(mycp->pt->x, mycp->pt->y),
+              thvec2(cp->pt->x, cp->pt->y));
+            } else {
+              xl = thline2(
+              thvec2(cp->pt->x, cp->pt->y),
+              thvec2(mycp->pt->x, mycp->pt->y));
+            }
+            
+            ccount++;
+            switch (this->fscrapptr->proj->type) {
+              case TT_2DPROJ_ELEV:
+              case TT_2DPROJ_EXTEND:
+                if (this->point->y > mycp->pt->y) {
+                  if (!reverse)
+                    cdist += cl->from_up;
+                  else
+                    cdist += cl->to_up;
+                } else {
+                  if (!reverse)
+                    cdist += cl->from_down;
+                  else
+                    cdist += cl->to_down;
+                }
+                break;
+              default:
+                if (xl.eval(thvec2(this->point->x, this->point->y)) < 0.0) {
+                  if (!reverse)
+                    cdist += cl->from_left;
+                  else
+                    cdist += cl->to_left;
+                } else {
+                  if (!reverse)
+                    cdist += cl->from_right;
+                  else
+                    cdist += cl->to_right;
+                }
+            }
+          }
+          carrow = carrow->next_arrow;
+        }
+      }
+      cp = cp->nextcp;
+    }
+    if (ccount > 0)
+      this->xsize = cdist / double(ccount);
+  }
+}
+
+
+
+
 
