@@ -39,6 +39,8 @@
 #include "thscrapis.h"
 #include "thsurvey.h"
 #include "thsymbolset.h"
+#include "thsketch.h"
+#include "thcsdata.h"
 
 #ifdef THMSVC
 #define hypot _hypot
@@ -146,8 +148,12 @@ thcmd_option_desc thscrap::get_cmd_option_desc(char * opts)
   int id = thmatch_token(opts, thtt_scrap_opt);
   if (id == TT_SCRAP_UNKNOWN)
     return thdataobject::get_cmd_option_desc(opts);
-  else
-    return thcmd_option_desc(id);
+  else switch (id) {
+    case TT_SCRAP_SKETCH:
+      return thcmd_option_desc(id,3);
+    default:
+      return thcmd_option_desc(id);
+  }
 }
 
 
@@ -166,6 +172,7 @@ void thscrap::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
 {
   
   thdb2dprjpr projection;
+
   if (cod.id == 1)
     cod.id = TT_DATAOBJECT_NAME;
     
@@ -204,6 +211,10 @@ void thscrap::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
       if (this->flip == TT_SCRAP_FLIP_UNKNOWN)
         ththrow(("invalid -flip switch -- %s", *args));
       break;
+
+    case TT_SCRAP_SKETCH:
+      this->parse_sketch(args, argenc);
+      break;
     
     // if not found, try to set fathers properties  
     default:
@@ -259,55 +270,56 @@ void thscrap::parse_scale(char * ss)
     thparse_double(sv,var,pars[parn]); \
     if (sv != TT_SV_NUMBER) \
       ththrow(("real number required -- %s", pars[parn]));
+  
+  if (ux > 0) {
+    lentf.parse_units(pars[ux]);
+  }
 
   if (p9) {
 //    thparse_double(sv,this->scale_r1x,pars[0]);
 //    if ((sv != TT_SV_NUMBER) || (n1 <= 0.0))
 //      ththrow(("real positive number required -- %s", pars[0]));
-      parse_scalep9(this->scale_p1x,0)
-      parse_scalep9(this->scale_p1y,1)
-      parse_scalep9(this->scale_p2x,2)
-      parse_scalep9(this->scale_p2y,3)
+    parse_scalep9(this->scale_p1x,0)
+    parse_scalep9(this->scale_p1y,1)
+    parse_scalep9(this->scale_p2x,2)
+    parse_scalep9(this->scale_p2y,3)
+    if (this->cs == TTCS_LOCAL) {
       parse_scalep9(this->scale_r1x,4)
       parse_scalep9(this->scale_r1y,5)
       parse_scalep9(this->scale_r2x,6)
       parse_scalep9(this->scale_r2y,7)
-      if (hypot(this->scale_r1x - this->scale_r2x,
-        this->scale_r1y - this->scale_r2y) == 0.0)
-        ththrow(("zero scale real length"));
-      if (hypot(this->scale_p1x - this->scale_p2x,
-        this->scale_p1y - this->scale_p2y) == 0.0)
-        ththrow(("zero scale picture length"));        
-  }
-  else {
+      if (ux > 0) {
+        this->scale_r1x = lentf.transform(this->scale_r1x);
+        this->scale_r1y = lentf.transform(this->scale_r1y);
+        this->scale_r2x = lentf.transform(this->scale_r2x);
+        this->scale_r2y = lentf.transform(this->scale_r2y);
+      }
+    } else {
+      this->convert_cs(pars[4], pars[5], this->scale_r1x, this->scale_r1y);
+      this->convert_cs(pars[6], pars[7], this->scale_r2x, this->scale_r2y);
+    }
+    if (hypot(this->scale_r1x - this->scale_r2x,
+      this->scale_r1y - this->scale_r2y) == 0.0)
+      ththrow(("zero scale real length"));
+    if (hypot(this->scale_p1x - this->scale_p2x,
+      this->scale_p1y - this->scale_p2y) == 0.0)
+      ththrow(("zero scale picture length"));        
+  } else {
     // let's parse first number
     thparse_double(sv,n1,pars[0]);
     if ((sv != TT_SV_NUMBER) || (n1 <= 0.0))
       ththrow(("real positive number required -- %s", pars[0]));
-      
     // let's parse second number
     if (n2x > 0) {
       thparse_double(sv,n2,pars[n2x]);
       if ((sv != TT_SV_NUMBER) || (n2 <= 0.0))
         ththrow(("real positive number required -- %s", pars[0]));
       n1 = n2 / n1;
+      if (ux > 0)
+        n1 = lentf.transform(n1);
     }
+    this->scale = n1;
   }    
-  // let's parse units
-  if (ux > 0) {
-    lentf.parse_units(pars[ux]);
-    if (p9) {
-      this->scale_r1x = lentf.transform(this->scale_r1x);
-      this->scale_r1y = lentf.transform(this->scale_r1y);
-      this->scale_r2x = lentf.transform(this->scale_r2x);
-      this->scale_r2y = lentf.transform(this->scale_r2y);
-    } 
-    else {
-      n1 = lentf.transform(n1);
-    }
-  }
-  
-  this->scale = n1;
 }
 
 thdb2dcp * thscrap::insert_control_point()
@@ -487,15 +499,25 @@ thscraplo * thscrap::get_outline() {
 } // get_outline
 
 
-thscraplp * thscrap::polygon_insert() {
+thscraplp * thscrap::polygon_insert(thscraplp * before) {
   thscraplp * rv = this->db->db2d.insert_scraplp();
-  if (this->polygon_last == NULL) {
-    this->polygon_first = rv;
-    this->polygon_last = rv;
+  if (before == NULL) {
+    if (this->polygon_last == NULL) {
+      this->polygon_first = rv;
+      this->polygon_last = rv;
+    } else {
+      this->polygon_last->next_item = rv;
+      rv->prev_item = this->polygon_last;
+      this->polygon_last = rv;
+    }
   } else {
-    this->polygon_last->next_item = rv;
-    rv->prev_item = this->polygon_last;
-    this->polygon_last = rv;
+    if (before->prev_item == NULL)
+      this->polygon_first = rv;
+    else
+      before->prev_item->next_item = rv;
+    rv->prev_item = before->prev_item;
+    before->prev_item = rv;
+    rv->next_item = before;
   }
   return rv;
 }
@@ -534,7 +556,7 @@ thscraplp * thscrap::get_polygon() {
   thdb2dcp * cp;
   thscraplp * lp, * nlp;
   thdb1ds * st, * st1, * st2;
-  unsigned long i, ni = this->db->db1d.station_vec.size(),
+  size_t i, ni = this->db->db1d.station_vec.size(),
     nlegs = this->db->db1d.leg_vec.size();
   thdataleg * cl;
   bool ffselect, ttselect;
@@ -558,14 +580,16 @@ thscraplp * thscrap::get_polygon() {
         }
         set_ffst_ttst;
         ffselect = ((this->centerline_survey == NULL) && (ffst->survey->selected));
-        ttselect = ((this->centerline_survey == NULL) && (ffst->survey->selected));
+        ttselect = ((this->centerline_survey == NULL) && (ttst->survey->selected));
         ffselect = ffselect || ((this->centerline_survey != NULL) && 
           (ffst->survey->is_in_survey(this->centerline_survey)));
         ttselect = ttselect || ((this->centerline_survey != NULL) && 
           (ttst->survey->is_in_survey(this->centerline_survey)));          
-        if (ffselect && ttselect && ((cl->extend & TT_EXTENDFLAG_IGNORE) == 0)) {
-          ffst->tmpselect = true;
-          ttst->tmpselect = true;
+        if ((cl->extend & TT_EXTENDFLAG_HIDE) == 0) {
+          if ((ffselect && ttselect) || (cl->psurvey->is_in_survey(this->centerline_survey))) {
+            ffst->tmpselect = true;
+            ttst->tmpselect = true;
+          }
         }
       }
     } else {
@@ -576,6 +600,14 @@ thscraplp * thscrap::get_polygon() {
           st->tmpselect = true;
         }
       }
+      for (i = 0; i < nlegs; i++) {
+        cl = this->db->db1d.leg_vec[i].leg;
+        //if (((cl->extend & TT_EXTENDFLAG_HIDE) == 0) && (cl->psurvey->is_in_survey(this->centerline_survey))) {
+        if ((cl->psurvey->is_in_survey(this->centerline_survey))) {
+          this->db->db1d.station_vec[cl->from.id - 1].tmpselect = true;
+          this->db->db1d.station_vec[cl->to.id - 1].tmpselect = true;
+        }
+      }
     } // other projections
 
     // enter selected stations
@@ -585,6 +617,9 @@ thscraplp * thscrap::get_polygon() {
         lp = this->polygon_insert();
         lp->station = st;
         lp->ustation = st;
+        lp->station_name.id = (unsigned long)i+1;
+        lp->station_name.name = st->name;
+        lp->station_name.psurvey = st->survey;
         this->insert_adata(st);
       }
     }  
@@ -609,31 +644,38 @@ thscraplp * thscrap::get_polygon() {
               switch (this->proj->type) {
                 case TT_2DPROJ_EXTEND:
                   cnode = &(nodes[st->uid - 1]);
-                  arrow = cnode->first_arrow;
-                  lp->stx = thnan;
+                  lp->stx = cnode->extendx - this->proj->shift_x;
                   lp->sty = st->z - this->proj->shift_z;
                   lp->stz = 0.0;
+                  arrow = cnode->first_arrow;
                   while (arrow != NULL) {
-                    if ((arrow->leg->leg->extend & TT_EXTENDFLAG_IGNORE) == 0) {
-                      newx = (arrow->is_reversed ? arrow->leg->leg->txx : arrow->leg->leg->fxx) - this->proj->shift_x;
-                      if (thisnan(lp->stx)) {
-                        lp->stx = newx;
-                        lp->sty = st->z - this->proj->shift_z;
-                        lp->stz = 0.0;
-                      } else if (incp && (lp->stx != newx)) {
-                        nlp = this->polygon_insert();
-                        nlp->station = st;
-                        nlp->ustation = st;
-                        nlp->stx = newx;
-                        nlp->sty = st->z - this->proj->shift_z;
-                        nlp->stz = 0.0;
-                      }
+                    newx = (arrow->is_reversed ? arrow->leg->leg->txx : arrow->leg->leg->fxx) - this->proj->shift_x;
+
+                    if (((arrow->leg->leg->extend & TT_EXTENDFLAG_HIDE) == 0) && (newx != lp->stx)) {
+                      nlp = this->polygon_insert(lp);
+                      nlp->station = st;
+                      nlp->ustation = st;
+                      nlp->station_name.id = st->uid;
+                      nlp->station_name.name = st->name;
+                      nlp->station_name.psurvey = st->survey;
+                      nlp->stx = newx - this->proj->shift_x;
+                      nlp->sty = st->z - this->proj->shift_z;
+                      nlp->stz = 0.0;
+                      thscraplp * olp;
+                      olp = nlp;
+
+                      nlp = this->polygon_insert(lp);
+                      nlp->lnio = true;
+                      nlp->type = SYML_MAPCONNECTION;
+                      nlp->lnx1 = lp->stx;
+                      nlp->lny1 = lp->sty;
+                      nlp->lnz1 = lp->stz;
+                      nlp->lnx2 = olp->stx;
+                      nlp->lny2 = olp->sty;
+                      nlp->lnz2 = olp->stz;
                     }
                     arrow = arrow->next_arrow;
                   }
-                  if (thisnan(lp->stx)) {
-                    lp->stx = cnode->xx - this->proj->shift_x;
-                  }
                   break;
                 case TT_2DPROJ_PLAN:
                   lp->stx = st->x - this->proj->shift_x;
@@ -659,9 +701,13 @@ thscraplp * thscrap::get_polygon() {
             arrow = nodes[st->uid - 1].first_arrow;
             while (arrow != NULL) {
             
-              if ((((this->centerline_survey != NULL) && (arrow->leg->survey->is_in_survey(this->centerline_survey))) || 
-                  ((this->centerline_survey == NULL) && (arrow->leg->survey->selected))) &&
-                  ((this->proj->type != TT_2DPROJ_EXTEND) || ((arrow->leg->leg->extend & TT_EXTENDFLAG_IGNORE) == 0))) {
+//              if ((((this->centerline_survey != NULL) && (arrow->leg->survey->is_in_survey(this->centerline_survey))) || 
+//                  ((this->centerline_survey == NULL) && (arrow->leg->survey->selected))) &&
+//                  ((this->proj->type != TT_2DPROJ_EXTEND) || ((arrow->leg->leg->extend & TT_EXTENDFLAG_IGNORE) == 0))) {
+              if (((((this->centerline_survey != NULL) && (arrow->leg->survey->is_in_survey(this->centerline_survey))) || 
+                  ((this->centerline_survey == NULL) && (arrow->leg->survey->selected)))) &&
+			      ((this->proj->type != TT_2DPROJ_EXTEND) || 
+				  ((arrow->leg->leg->extend & TT_EXTENDFLAG_HIDE) == 0))) {
 
                 // process arrow here
                 nlp = this->polygon_insert();
@@ -1251,3 +1297,39 @@ void thscrap::update_limits(double x, double y)
 }
 
 
+
+void thscrap::parse_sketch(char ** args, int argenc)
+{
+  int sv;
+  thsketch sk;
+  sk.m_scrap = this;
+  sk.m_pic.init(args[0], thdb.csrc.name);
+  // X
+  thparse_double(sv,sk.m_x,args[1]);
+  if ((sv	!= TT_SV_NUMBER) &&	(sv	!= TT_SV_NAN))
+    ththrow(("invalid	number --	%s", args[1]))
+  // Y
+  thparse_double(sv,sk.m_y,args[2]);
+  if ((sv	!= TT_SV_NUMBER) &&	(sv	!= TT_SV_NAN))
+    ththrow(("invalid	number --	%s", args[2]))
+  this->sketch_list.push_back(sk);
+}
+
+
+void thscrap::start_insert() {
+  if (this->cs != TTCS_LOCAL) {
+    if (this->proj->type != TT_2DPROJ_PLAN)
+      ththrow(("coordinate system specification valid only for plan projection"))
+    if (!this->scale_p9)
+      ththrow(("scrap scaling not valid in this coordinate system"))
+  }
+}
+
+
+
+
+
+
+
+
+
